local addon = PvPTimer

local AceGUI = addon.Lib.AceGUI
local L = addon.Locale
local ST = LibStub("ScrollingTable");

local type = type
local unpack = unpack
local pairs = pairs
local tonumber = tonumber
local sformat = string.format
local GetSpellInfo = GetSpellInfo
local select = select

local const_racial = addon.Const.Racial
local const_item = addon.Const.Item
local const_on = addon.Const.On
local const_off = addon.Const.Off
local tt = PT_Tooltip

local classes = addon.Const.Classes
local classFilter = {}
for k in pairs(classes) do
	classFilter[k] = true
end
local classFilterIcons = {}

local function _GetSpellIcon(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, self, ...)
	if fShow then 
		local spellID = tonumber(data[realrow].cols[column].value)
		local texture = select(3, GetSpellInfo(spellID))
		cellFrame.text:SetText("|T"..texture..":16:16|t")
	end				
end

local function _GetSpellName(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, self, ...)
	if fShow then
		local text = data[realrow].cols[column].value
		local id = data[realrow].cols[1].value
		local db = addon.DB.profile.Spells
		
		if db[id] and db[id].enabled == false then
			cellFrame.text:SetText("|c80FF0000"..text)
		else
			cellFrame.text:SetText("|CFFFFFFFF"..text)
		end
	end
end

local function _GetClass(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, self, ...)
	if fShow then 
		local class = data[realrow].cols[column].value
		if classes[class] then
			local color = RAID_CLASS_COLORS[class]
			local text = sformat("|cFF%02x%02x%02x%s", color.r*255, color.g*255, color.b*255, classes[class])
			cellFrame.text:SetText(text)
		elseif class == "RACIAL" then
			cellFrame.text:SetText(const_racial)
		else
			cellFrame.text:SetText(const_item)
		end
	end				
end

local function _GetGlyph(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, self, ...)
	if fShow then
		local id = data[realrow].cols[1].value
		local glyph = data[realrow].cols[column].value
		local class = data[realrow].cols[3].value
		local spell = addon.Spells[id]
		local spec
		if spell.spec then
			spec = spell.spec
		else
			spec = 0
		end
		if glyph == 0 or class == "ITEM" or class == "RACIAL" or (spec ~=0 and column - spec ~= 7) then
			cellFrame.text:SetText("|c80808080N/A")
			cellFrame.text:SetJustifyH("CENTER")
			return
		end

		local db = addon.DB.profile.Spells
		local id = data[realrow].cols[1].value
		local specs = addon.Const.Specs
		local text
		if column == 7 then
			text = "|T"..specs["default"].icon..":16:16|t "
		else
			text = "|T"..specs[class][column-7].icon..":16:16|t "
		end
		
		if (column == 7 and db[id] and db[id].glyph0) or
			(column == 8 and db[id] and db[id].glyph1) or
			(column == 9 and db[id] and db[id].glyph2) or
			(column == 10 and db[id] and db[id].glyph3) then
			text = text..const_on
		else
			text = text..const_off
		end
		
		cellFrame.text:SetJustifyV("CENTER")
		cellFrame.text:SetText(text)
	end				
end

local spellST
local spellTableCols = {
	{ name = "", width = 20, defaultsort = "dsc", DoCellUpdate = _GetSpellIcon, },
	{ name = L["Name"], width = 155, defaultsort = "dsc", sort = "dsc", DoCellUpdate = _GetSpellName},
	{ name = L["Class"], width = 80, defaultsort = "dsc", DoCellUpdate = _GetClass, },
	{ name = L["Category"], width = 70, defaultsort = "dsc", },
	{ name = L["Duration"], width = 60, defaultsort = "dsc", },
	{ name = L["Cooldown"], width = 60, defaultsort = "dsc", },
	{ name = L["Glyph"], width = 50, defaultsort = "dsc", DoCellUpdate = _GetGlyph},
	{ name = L["Spec 1"], width = 50, defaultsort = "dsc", DoCellUpdate = _GetGlyph},
	{ name = L["Spec 2"], width = 50, defaultsort = "dsc", DoCellUpdate = _GetGlyph},
	{ name = L["Spec 3"], width = 50, defaultsort = "dsc", DoCellUpdate = _GetGlyph},
}
local spellTableRows = 33
local spellTableRowHeight = 16

local COL_ID = 1
local COL_NAME = 2
local COL_CLASS = 3
local COL_TYPE = 4
local COL_DUR = 5
local COL_CD = 6
local COL_G0 = 7
local COL_G1 = 8
local COL_G2 = 9
local COL_G3 = 10

local function GetSpellTableData()
	local const_stypes = addon.DB.profile.SpellCategories
	
	local src = addon.Spells
	local sd = {}
	local glyphed = ""

	for k, v in pairs(src) do
		if type(v) == "table" then
			sd[#sd+1] = {
				cols = {
					{--ID
						value = k,
					},
					{--Name
						value = v.name,
					},
					{--Class
						value = v.class or "ITEM",
					},
					{--Type
						value = v.type or "misc", 
						color = const_stypes[v.type] or const_stypes["misc"]
					},
					{--Duration
						value = v.duration or "---",
					},
					{--CD
						value = v.cooldown or "---",
					},
					{--Glyph
						value = v.cooldown_g or 0,
					},
					{--Glyph
						value = v.cooldown_g or 0,
					},
					{--Glyph
						value = v.cooldown_g or 0,
					},
					{--Glyph
						value = v.cooldown_g or 0,
					},
				},
			}
		end
	end
	
	return sd
end

local function ActivateControl(widget, event, button, ...)
	local data = widget:GetUserDataTable().id
	classFilter[data] = not classFilter[data]
	if classFilter[data] then
		classFilterIcons[data]:SetLabel(classes[data])
	else
		classFilterIcons[data]:SetLabel("|c80808080"..classes[data])
	end
	spellST:SortData()
end

local function SpellFilter(self, row)
	local class = row.cols[3].value
	if class == "RACIAL" or class == "ITEM" or classFilter[class] then
		return true
	end
	return false
end

local function TableEnter(rowFrame, cellFrame, data, cols, row, realrow, column, table, ...)
	if not realrow then return end
	local spell = tonumber(data[realrow].cols[1].value)
	local glyph = tonumber(data[realrow].cols[7].value)
	if spell>0 then
		local db = addon.DB.profile.Spells
		if not db[spell] then db[spell] = {} end
		db = db[spell]
		
		local class = data[realrow].cols[3].value
		local cooldown = tonumber(data[realrow].cols[6].value)
		local specs = addon.Const.Specs
		local spelldata = addon.Spells[spell]

		local spec
		if spelldata.spec then
			spec = spelldata.spec
		else
			spec = 0
		end
		
		tt:SetOwner(cellFrame, "ANCHOR_TOPLEFT", -5, 5)
		tt:SetSpellByID(spell)
		
		if class == "RACIAL" and column == 2 then
			tt:AddLine("\n"..L["Click here to enable/disable this racial."], 0, 1, 0, true)
			tt:Show()
			return
		end
		if class == "ITEM" and column == 2 then
			tt:AddLine("\n"..L["Click here to enable/disable this item."], 0, 1, 0, true)
			tt:Show()
			return
		end

		if column == 7 and glyph ~= 0 then
			tt:AddLine("\n"..L["Click here to enable/disable the glyph of this spell when the unit's talent spec is unknown. This should make the cooldown timers more accurate."], 0, 1, 0, true)
		elseif column >= 8 and column <= 10 and data[realrow].cols[column].value ~= 0 then
			tt:AddLine("\n"..L["Click here to enable/disable the glyph of this spell for this talent spec. This should make the cooldown timers more accurate."], 0, 1, 0, true)
		elseif column == 2 then
			tt:AddLine("\n"..L["Click here to enable/disable this spell."], 0, 1, 0, true)
		end

		tt:AddLine("\n"..L["Base cooldown: "]..addon:ConvertTime(cooldown), 1, 1, 1, false)

		if glyph and glyph ~= 0 then
			tt:AddLine(L["Glyph modifier: "]..addon:ConvertTime(glyph), 1, 1, 1, false)
		end
		
		if specs[class] then
			local line = ""
			if spec == 0 then line = line.."\n"..L["Unknown"]..": "..addon:ConvertTime(addon:GetSpellCooldown(spell, 0)) end
			if spec == 0 or spec == 1 then line = line.."\n"..specs[class][1].name..": "..addon:ConvertTime(addon:GetSpellCooldown(spell, 1)) end
			if spec == 0 or spec == 2 then line = line.."\n"..specs[class][2].name..": "..addon:ConvertTime(addon:GetSpellCooldown(spell, 2)) end
			if spec == 0 or spec == 3 then line = line.."\n"..specs[class][3].name..": "..addon:ConvertTime(addon:GetSpellCooldown(spell, 3)) end
			tt:AddLine(line, 1, 1, 1, true)
		end

		tt:Show()
	else
		tt:Hide()
	end
end

local function TableClick(rowFrame, cellFrame, data, cols, row, realrow, column, table, button, ...)
	-- table header clicked, do sort
	if not realrow then
		table.DefaultEvents["OnClick"](rowFrame, cellFrame, data, cols, row, realrow, column, table, button, ...)
	-- glyph settings
	elseif column >= 7 and column <= 10 and glyph ~= 0 then
		local db = addon.DB.profile.Spells
		local id = data[realrow].cols[1].value
		local glyph = data[realrow].cols[7].value
		if not db[id] then db[id] = {} end
		if column == 7 then
			db[id].glyph0 = not db[id].glyph0
		elseif column == 8 then
			db[id].glyph1 = not db[id].glyph1
		elseif column == 9 then
			db[id].glyph2 = not db[id].glyph2
		elseif column == 10 then
			db[id].glyph3 = not db[id].glyph3
		end
	-- enable/disable spell
	elseif column == 2 then
		local db = addon.DB.profile.Spells
		local id = data[realrow].cols[1].value
		if not db[id] or db[id].enabled == nil then
			db[id] = {}
			db[id].enabled = false
		else
			db[id].enabled = not db[id].enabled
		end
	end
	-- refresh tooltip
	TableEnter(rowFrame, cellFrame, data, cols, row, realrow, column, table, ...)
	-- refresh table
	table:Refresh()
end

function addon:CreateSpellConfig()
	addon.Window_SpellConfig = AceGUI:Create("Window")
	local f = addon.Window_SpellConfig

	f:SetCallback("OnClose", function() f:Hide() end)
	f:SetTitle(L["PvPTimer Spell Configuration"])
	f:SetLayout("Flow")
	f:SetWidth(800)
	f:SetHeight(600)
	f:SetPoint("CENTER", "UIParent", "CENTER")
	f:EnableResize(false)
	
	spellST = ST:CreateST(spellTableCols, spellTableRows, spellTableRowHeight, nil, f.frame)
	spellST:SetData(GetSpellTableData())
	spellST:SetFilter(SpellFilter)
	spellST:RegisterEvents({
		["OnClick"] = TableClick,
		["OnEnter"] = TableEnter,
		["OnLeave"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
			tt:Hide()
		end,
	})
	spellST.frame:SetPoint("BOTTOMRIGHT", f.frame, "BOTTOMRIGHT", -8, 10)

	local filterClass = AceGUI:Create("InlineGroup")
		filterClass:SetTitle("Class Filter")
		filterClass:SetLayout("Flow")
		filterClass:SetWidth(100)
	f:AddChild(filterClass)
	
	for k, v in pairs(classes) do
		local icon = AceGUI:Create("Icon")

		local left, right, top, bottom = unpack(CLASS_BUTTONS[k])
		left = left + (right - left) * 0.07
		right = right - (right - left) * 0.07
		top = top + (bottom - top) * 0.07
		bottom = bottom - (bottom - top) * 0.07

		icon:SetImage("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes", left, right, top, bottom)
		icon:SetImageSize(24, 24)
		icon:SetLabel(v)
		icon:SetUserData("id", k)
		
		icon:SetCallback("OnClick", ActivateControl)
		
		filterClass:AddChild(icon)
		classFilterIcons[k] = icon
	end
	filterClass:DoLayout()

	f:Hide()
end

function addon:ToggleSpellConfig()
	if not addon.Window_SpellConfig then
		addon:CreateSpellConfig()
	end
	local f = addon.Window_SpellConfig

	if f and f:IsVisible() then
		f:Hide()
	else
		GameTooltip:Hide()
		f:Show()
	end
end
